@@ -30,6 +30,18 @@
 #include "tracker-miner-dbus.h"
 #include "tracker-miner-glue.h"
 
+/**
+ * SECTION:tracker-miner
+ * @short_description: Abstract base class for data miners
+ * @include: libtracker-miner/tracker-miner.h
+ *
+ * #TrackerMiner is an abstract base class to help developing data miners
+ * for tracker-store, being an abstract class it doesn't do much by itself,
+ * but provides the basic signaling and operation control so the miners
+ * implementing this class are properly recognized by Tracker, and can be
+ * controlled properly by external means such as #TrackerMinerManager.
+ **/
+
 #define TRACKER_MINER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER, TrackerMinerPrivate))
 
 struct TrackerMinerPrivate {
@@ -109,6 +121,14 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	object_class->finalize     = miner_finalize;
 	object_class->constructed  = miner_constructed;
 
+	/**
+	 * TrackerMiner::started:
+	 * @miner: the #TrackerMiner
+	 *
+	 * the ::started signal is emitted in the miner
+	 * right after it has been started through
+	 * tracker_miner_start().
+	 **/
 	signals[STARTED] =
 		g_signal_new ("started",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -117,6 +137,14 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	/**
+	 * TrackerMiner::stopped:
+	 * @miner: the #TrackerMiner
+	 *
+	 * the ::stopped signal is emitted in the miner
+	 * right after it has been stopped through
+	 * tracker_miner_stop().
+	 **/
 	signals[STOPPED] =
 		g_signal_new ("stopped",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -125,6 +153,15 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	/**
+	 * TrackerMiner::paused:
+	 * @miner: the #TrackerMiner
+	 *
+	 * the ::paused signal is emitted whenever
+	 * there is any reason to pause, either
+	 * internal (through tracker_miner_pause()) or
+	 * external (through DBus, see #TrackerMinerManager).
+	 **/
 	signals[PAUSED] =
 		g_signal_new ("paused",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -133,6 +170,14 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	/**
+	 * TrackerMiner::resumed:
+	 * @miner: the #TrackerMiner
+	 *
+	 * the ::resumed signal is emitted whenever
+	 * all reasons to pause have disappeared, see
+	 * tracker_miner_resume() and #TrackerMinerManager.
+	 **/
 	signals[RESUMED] =
 		g_signal_new ("resumed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -149,6 +194,17 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	/**
+	 * TrackerMiner::progress:
+	 * @miner: the #TrackerMiner
+	 * @status: miner status
+	 * @progress: a #gdouble indicating miner progress, from 0 to 1.
+	 *
+	 * the ::progress signal will be emitted by TrackerMiner implementations
+	 * to indicate progress about the data mining process. @status will
+	 * contain a translated string with the current miner status and @progress
+	 * will indicate how much has been processed so far.
+	 **/
 	signals[PROGRESS] =
 		g_signal_new ("progress",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -172,23 +228,23 @@ tracker_miner_class_init (TrackerMinerClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
 					 g_param_spec_string ("name",
-							      "Name",
-							      "Name",
+							      "Miner name",
+							      "Miner name",
 							      NULL,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_STATUS,
 					 g_param_spec_string ("status",
 							      "Status",
-							      "Status (unique to each miner)",
+							      "Translatable string with status description",
 							      NULL,
 							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_PROGRESS,
 					 g_param_spec_double ("progress",
 							      "Progress",
-							      "Progress (between 0 and 1)",
-							      0.0, 
+							      "Miner progress",
+							      0.0,
 							      1.0,
 							      0.0,
 							      G_PARAM_READWRITE));
@@ -447,7 +503,6 @@ name_owner_changed_cb (DBusGProxy *proxy,
 		gint cookie_id;
 
 		cookie_id = tracker_miner_pause (miner,
-						 g_get_application_name (),
 						 _("Data store is not available"),
 						 &error);
 
@@ -779,20 +834,16 @@ tracker_miner_commit (TrackerMiner *miner)
 	return TRUE;
 }
 
-gint
-tracker_miner_pause (TrackerMiner  *miner,
-		     const gchar   *application,
-		     const gchar   *reason,
-		     GError       **error)
+static gint
+tracker_miner_pause_internal (TrackerMiner  *miner,
+			      const gchar   *application,
+			      const gchar   *reason,
+			      GError       **error)
 {
 	PauseData *pd;
 	GHashTableIter iter;
 	gpointer key, value;
 
-	g_return_val_if_fail (TRACKER_IS_MINER (miner), -1);
-	g_return_val_if_fail (application != NULL, -1);
-	g_return_val_if_fail (reason != NULL, -1);
-
 	/* Check this is not a duplicate pause */
 	g_hash_table_iter_init (&iter, miner->private->pauses);
 	while (g_hash_table_iter_next (&iter, &key, &value)) {
@@ -822,6 +873,37 @@ tracker_miner_pause (TrackerMiner  *miner,
 	return pd->cookie;
 }
 
+/**
+ * tracker_miner_pause:
+ * @miner: a #TrackerMiner
+ * @reason: reason to pause
+ * @error: return location for errors
+ *
+ * Asks @miner to pause. On success the cookie ID is returned,
+ * this is what must be used in tracker_miner_resume() to resume
+ * operations. On failure @error will be set and -1 will be returned.
+ *
+ * Returns: The pause cookie ID.
+ **/
+gint
+tracker_miner_pause (TrackerMiner  *miner,
+		     const gchar   *reason,
+		     GError       **error)
+{
+	const gchar *application;
+
+	g_return_val_if_fail (TRACKER_IS_MINER (miner), -1);
+	g_return_val_if_fail (reason != NULL, -1);
+
+	application = g_get_application_name ();
+
+	if (!application) {
+		application = miner->private->name;
+	}
+
+	return tracker_miner_pause_internal (miner, application, reason, error);
+}
+
 /**
  * tracker_miner_resume:
  * @miner: a #TrackerMiner
@@ -960,7 +1042,7 @@ tracker_miner_dbus_pause (TrackerMiner           *miner,
 				  application,
 				  reason);
 
-	cookie = tracker_miner_pause (miner, application, reason, &local_error);
+	cookie = tracker_miner_pause_internal (miner, application, reason, &local_error);
 	if (cookie == -1) {
 		GError *actual_error = NULL;
 
