@@ -44,12 +44,12 @@
 #endif
 
 #include <libtracker-common/tracker-log.h>
-#include <libtracker-common/tracker-config.h>
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-os-dependant.h>
-#include <libtracker-common/tracker-thumbnailer.h>
 #include <libtracker-common/tracker-ioprio.h>
 
+#include "tracker-albumart.h"
+#include "tracker-config.h"
 #include "tracker-main.h"
 #include "tracker-dbus.h"
 #include "tracker-extract.h"
@@ -68,12 +68,13 @@
 
 static GMainLoop  *main_loop;
 static guint       quit_timeout_id = 0;
-static TrackerHal *hal;
 
 static gboolean    version;
+static gboolean    disable_shutdown;
 static gint        verbosity = -1;
 static gchar      *filename;
 static gchar      *mime_type;
+static TrackerFTSConfig *fts_config;
 
 static GOptionEntry  entries[] = {
 	{ "version", 'V', 0,
@@ -89,10 +90,16 @@ static GOptionEntry  entries[] = {
 	  G_OPTION_ARG_FILENAME, &filename,
 	  N_("File to extract metadata for"),
 	  N_("FILE") },
-	{ "file", 'm', 0,
+	{ "mime", 'm', 0,
 	  G_OPTION_ARG_STRING, &mime_type,
 	  N_("MIME type for file (if not provided, this will be guessed)"),
 	  N_("MIME") },
+	/* Debug run is used to avoid that the mainloop exits, so that
+	 * as a developer you can be relax when running the tool in gdb */
+	{ "disable-shutdown", 'd', 0,
+	  G_OPTION_ARG_NONE, &disable_shutdown,
+	  N_("Disable shutting down after 30 seconds of inactivity"),
+	  NULL },
 
 	{ NULL }
 };
@@ -101,7 +108,12 @@ static gboolean
 quit_timeout_cb (gpointer user_data)
 {
 	quit_timeout_id = 0;
-	g_main_loop_quit (main_loop);
+	
+	if (!disable_shutdown) {
+		g_main_loop_quit (main_loop);
+	} else {
+		g_debug ("Would have quit the mainloop");
+	}
 
 	return FALSE;
 }
@@ -118,20 +130,6 @@ tracker_main_quit_timeout_reset (void)
 						 NULL);
 }
 
-TrackerHal *
-tracker_main_get_hal (void)
-{
-	if (!hal) {
-#ifdef HAVE_HAL
-		hal = tracker_hal_new ();
-#else 
-		hal = NULL;
-#endif
-	}
-
-	return hal;
-}
-
 static void
 initialize_priority (void)
 {
@@ -185,17 +183,19 @@ signal_handler (int signo)
 	}
 
 	switch (signo) {
+	case SIGABRT:
 	case SIGALRM:
-		exit (EXIT_FAILURE);
+		_exit (EXIT_FAILURE);
 		break;
 	case SIGTERM:
 	case SIGINT:
 		in_loop = TRUE;
+		disable_shutdown = FALSE;
 		quit_timeout_cb (NULL);
 	default:
 		if (g_strsignal (signo)) {
 			g_print ("\n");
-			g_print ("Received signal:%d->'%s'",
+			g_print ("Received signal:%d->'%s'\n",
 				 signo,
 				 g_strsignal (signo));
 		}
@@ -219,6 +219,7 @@ initialize_signal_handler (void)
 	sigaction (SIGINT,  &act, NULL);
 	sigaction (SIGHUP,  &act, NULL);
 	sigaction (SIGALRM,  &act, NULL);
+	sigaction (SIGABRT, &act, NULL);
 #endif /* G_OS_WIN32 */
 }
 
@@ -247,22 +248,45 @@ log_handler (const gchar    *domain,
 	case G_LOG_LEVEL_INFO:
 	case G_LOG_LEVEL_DEBUG:
 	case G_LOG_LEVEL_MASK:
+	default:
 		g_fprintf (stdout, "%s\n", message);
 		fflush (stdout);
 		break;
 	}	
 }
 
+TrackerFTSConfig *
+tracker_main_get_fts_config (void)
+{
+	if (G_UNLIKELY (!fts_config)) {
+		fts_config = tracker_fts_config_new ();
+	}
+
+	return fts_config;
+}
+
 int
 main (int argc, char *argv[])
 {
 	GOptionContext *context;
 	GError         *error = NULL;
 	TrackerConfig  *config;
-	gchar          *log_filename;
+	gchar          *log_filename = NULL;
 	gboolean        stand_alone = FALSE;
 	guint           log_handler_id = 0;
 
+	g_type_init ();
+
+	if (!g_thread_supported ()) {
+		g_thread_init (NULL);
+	}
+
+	dbus_g_thread_init ();
+
+	g_set_application_name ("tracker-extract");
+
+	setlocale (LC_ALL, "");
+
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -299,23 +323,13 @@ main (int argc, char *argv[])
 	}
 
 	g_print ("Initializing tracker-extract...\n");
+	g_print ("  Shutdown after 30 seconds of inactivitiy is %s\n",
+		 disable_shutdown ? "disabled" : "enabled");
 
 	initialize_signal_handler ();
 
 	tracker_memory_setrlimits ();
 
-	g_type_init ();
-
-	if (!g_thread_supported ()) {
-		g_thread_init (NULL);
-	}
-
-	dbus_g_thread_init ();
-
-	g_set_application_name ("tracker-extract");
-
-	setlocale (LC_ALL, "");
-
 	/* Set conditions when we use stand alone settings */
 	stand_alone |= filename != NULL;
 
@@ -335,35 +349,29 @@ main (int argc, char *argv[])
 	}
 
 	if (filename) {
-		TrackerExtract *object;
+		TrackerExtract *extract;
 		GFile *file;
-		gchar *full_path;
+		gchar *uri;
 
-		object = tracker_extract_new ();
-		if (!object) {
+		extract = tracker_extract_new (disable_shutdown);
+		if (!extract) {
 			return EXIT_FAILURE;
 		}
 
 		file = g_file_new_for_commandline_arg (filename);
-		full_path = g_file_get_path (file);
+		uri = g_file_get_uri (file);
 
-		tracker_extract_get_metadata_by_cmdline (object, full_path, mime_type);
+		tracker_extract_get_metadata_by_cmdline (extract, uri, mime_type);
 
-		g_object_unref (object);
+		g_object_unref (extract);
 		g_object_unref (file);
-		g_free (full_path);
+		g_free (uri);
 
 		goto done;
 	}
 
 	config = tracker_config_new ();
 
-	log_filename =
-		g_build_filename (g_get_user_data_dir (),
-				  "tracker",
-				  "tracker-extract.log",
-				  NULL);
-
 	/* Extractor command line arguments */
 	if (verbosity > -1) {
 		tracker_config_set_verbosity (config, verbosity);
@@ -376,19 +384,20 @@ main (int argc, char *argv[])
 	/* Initialize subsystems */
 	initialize_directories ();
 
-	tracker_log_init (log_filename, tracker_config_get_verbosity (config));
+	tracker_log_init (tracker_config_get_verbosity (config),
+			  &log_filename);
 	g_print ("Starting log:\n  File:'%s'\n", log_filename);
-
-	tracker_thumbnailer_init (config);
+	g_free (log_filename);
 
 	/* Make Tracker available for introspection */
-	if (!tracker_dbus_register_objects ()) {
-		g_free (log_filename);
+	if (!tracker_dbus_register_objects (disable_shutdown)) {
 		g_object_unref (config);
 
 		return EXIT_FAILURE;
 	}
 
+	tracker_albumart_init ();
+
 	/* Main loop */
 	main_loop = g_main_loop_new (NULL, FALSE);
 	tracker_main_quit_timeout_reset ();
@@ -397,19 +406,11 @@ main (int argc, char *argv[])
 
 	g_message ("Shutdown started");
 
-	/* Push all items in thumbnail queue to the thumbnailer */
-	tracker_thumbnailer_queue_send ();
-
 	/* Shutdown subsystems */
+	tracker_albumart_shutdown ();
 	tracker_dbus_shutdown ();
-	tracker_thumbnailer_shutdown ();
 	tracker_log_shutdown ();
 
-	if (hal) {
-		g_object_unref (hal);
-	}
-
-	g_free (log_filename);
 	g_object_unref (config);
 
 done:
