@@ -33,12 +33,13 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 
+#include <libtracker-common/tracker-common.h>
+
 #include <libtracker-db/tracker-db-manager.h>
-#include <libtracker-db/tracker-db-index-manager.h>
 
 static gboolean     should_kill;
 static gboolean     should_terminate;
-static gboolean     hard_reset;
+static gboolean     hard_reset, soft_reset;
 
 static GOptionEntry entries[] = {
 	{ "kill", 'k', 0, G_OPTION_ARG_NONE, &should_kill,
@@ -48,6 +49,9 @@ static GOptionEntry entries[] = {
 	  N_("Use SIGTERM to stop all tracker processes found"),
 	  NULL 
 	},
+	{ "soft-reset", 's', 0, G_OPTION_ARG_NONE, &soft_reset,
+	  N_("This will kill all Tracker processes and remove all databases except the backup and journal (a restart will restore the data)"),
+	  NULL },
 	{ "hard-reset", 'r', 0, G_OPTION_ARG_NONE, &hard_reset,
 	  N_("This will kill all Tracker processes and remove all databases"),
 	  NULL },
@@ -110,6 +114,7 @@ log_handler (const gchar    *domain,
 	case G_LOG_LEVEL_INFO:
 	case G_LOG_LEVEL_DEBUG:
 	case G_LOG_LEVEL_MASK:
+	default:
 		g_fprintf (stdout, "%s\n", message);
 		fflush (stdout);
 		break;
@@ -144,19 +149,23 @@ main (int argc, char **argv)
 		g_printerr ("%s\n",
 			    _("You can not use the --kill and --terminate arguments together"));
 		return EXIT_FAILURE;
-	} else if (hard_reset && should_terminate) {
+	} else if ((hard_reset || soft_reset) && should_terminate) {
 		g_printerr ("%s\n",
 			    _("You can not use the --terminate with --hard-reset, --kill is implied"));
 		return EXIT_FAILURE;
 	}
 
-	if (hard_reset) {
+	if (hard_reset || soft_reset) {
 		/* Imply --kill */
 		should_kill = TRUE;
 	}
 
 	pids = get_pids ();
-	str = g_strdup_printf (_("Found %d pids..."), g_slist_length (pids));
+	str = g_strdup_printf (tracker_dngettext (NULL,
+						  "Found %d PID…", 
+						  "Found %d PIDs…",
+						  g_slist_length (pids)),
+			       g_slist_length (pids));
 	g_print ("%s\n", str);
 	g_free (str);
 
@@ -235,7 +244,7 @@ main (int argc, char **argv)
 	g_slist_foreach (pids, (GFunc) g_free, NULL);
 	g_slist_free (pids);
 
-	if (hard_reset) {
+	if (hard_reset || soft_reset) {
 		guint log_handler_id;
 
 		/* Set log handler for library messages */
@@ -251,16 +260,9 @@ main (int argc, char **argv)
 			return EXIT_FAILURE;
 		}
 
-		tracker_db_manager_remove_all ();
+		tracker_db_manager_remove_all (hard_reset);
 		tracker_db_manager_shutdown ();
 
-		if (!tracker_db_index_manager_init (TRACKER_DB_INDEX_MANAGER_REMOVE_ALL, 0, 0)) {
-			return EXIT_FAILURE;
-		}
-
-		tracker_db_index_manager_remove_all ();
-		tracker_db_index_manager_shutdown ();
-
 		/* Unset log handler */
 		g_log_remove_handler (NULL, log_handler_id);
 	}
